@@ -65,7 +65,7 @@ class EnergyDataset(DatasetLoaderCSV):
 class GasRateCO2Dataset(DatasetLoaderCSV):
     """
     Gas Rate CO2 dataset
-    Two components, length 296 (dummy time index)
+    Two components, length 296 (integer time index)
     """
 
     def __init__(self):
@@ -85,7 +85,7 @@ class HeartRateDataset(DatasetLoaderCSV):
     each series is exactly 15 minutes.
 
     This is the series1 here: http://ecg.mit.edu/time-series/
-    Using a dummy time index.
+    Using an integer time index.
     """
 
     def __init__(self):
@@ -169,7 +169,7 @@ class TaylorDataset(DatasetLoaderCSV):
     """
     Half-hourly electricity demand in England and Wales from Monday 5 June 2000 to Sunday 27 August 2000.
     Discussed in Taylor (2003) [1], and kindly provided by James W Taylor. Units: Megawatts
-    (Uses a dummy time index).
+    (Uses an integer time index).
 
     https://www.rdocumentation.org/packages/forecast/versions/8.13/topics/taylor
 
