@@ -46,6 +46,6 @@ from .baselines import NaiveEnsembleModel
 from .regression_ensemble_model import RegressionEnsembleModel
 
 # Filtering
-from .filtering_model import MovingAverage
+from .moving_average import MovingAverage
 from .kalman_filter import KalmanFilter
 from .gaussian_process_filter import GaussianProcessFilter
