@@ -3,7 +3,7 @@ N-BEATS
 -------
 """
 
-from typing import NewType, Union, List, Optional
+from typing import NewType, Union, List, Optional, Tuple
 from enum import Enum
 import numpy as np
 from numpy.random import RandomState
@@ -12,7 +12,7 @@ import torch.nn as nn
 
 from ..logging import get_logger, raise_log, raise_if_not
 from ..utils.torch import random_method
-from .torch_forecasting_model import TorchForecastingModel
+from .torch_forecasting_model import PastCovariatesTorchModel
 
 logger = get_logger(__name__)
 
@@ -37,7 +37,7 @@ class _TrendGenerator(nn.Module):
             False)
 
     def forward(self, x):
-        return torch.matmul(x, self.T.float().T)
+        return torch.matmul(x, self.T.T)
 
 
 class _SeasonalityGenerator(nn.Module):
@@ -52,7 +52,7 @@ class _SeasonalityGenerator(nn.Module):
                               False)
 
     def forward(self, x):
-        return torch.matmul(x, self.S.float().T)
+        return torch.matmul(x, self.S.T)
 
 
 class _Block(nn.Module):
@@ -357,7 +357,7 @@ class _NBEATSModule(nn.Module):
         return y
 
 
-class NBEATSModel(TorchForecastingModel):
+class NBEATSModel(PastCovariatesTorchModel):
     @random_method
     def __init__(self,
                  input_chunk_length: int,
@@ -380,6 +380,8 @@ class NBEATSModel(TorchForecastingModel):
         supports multivariate series (and covariates) by flattening the model inputs to a 1-D series
         and reshaping the outputs to a tensor of appropriate dimensions.
 
+        This model supports past covariates (known for `input_chunk_length` points before prediction time).
+
         Parameters
         ----------
         input_chunk_length
@@ -413,6 +415,46 @@ class NBEATSModel(TorchForecastingModel):
             Control the randomness of the weights initialization. Check this
             `link <https://scikit-learn.org/stable/glossary.html#term-random-state>`_ for more details.
 
+        batch_size
+            Number of time series (input and output sequences) used in each training pass.
+        n_epochs
+            Number of epochs over which to train the model.
+        optimizer_cls
+            The PyTorch optimizer class to be used (default: `torch.optim.Adam`).
+        optimizer_kwargs
+            Optionally, some keyword arguments for the PyTorch optimizer (e.g., `{'lr': 1e-3}`
+            for specifying a learning rate). Otherwise the default values of the selected `optimizer_cls`
+            will be used.
+        lr_scheduler_cls
+            Optionally, the PyTorch learning rate scheduler class to be used. Specifying `None` corresponds
+            to using a constant learning rate.
+        lr_scheduler_kwargs
+            Optionally, some keyword arguments for the PyTorch optimizer.
+        loss_fn
+            PyTorch loss function used for training.
+            This parameter will be ignored for probabilistic models if the `likelihood` parameter is specified.
+            Default: `torch.nn.MSELoss()`.
+        model_name
+            Name of the model. Used for creating checkpoints and saving tensorboard data. If not specified,
+            defaults to the following string "YYYY-mm-dd_HH:MM:SS_torch_model_run_PID", where the initial part of the
+            name is formatted with the local date and time, while PID is the processed ID (preventing models spawned at
+            the same time by different processes to share the same model_name). E.g.,
+            2021-06-14_09:53:32_torch_model_run_44607.
+        work_dir
+            Path of the working directory, where to save checkpoints and Tensorboard summaries.
+            (default: current working directory).
+        log_tensorboard
+            If set, use Tensorboard to log the different parameters. The logs will be located in:
+            `[work_dir]/.darts/runs/`.
+        nr_epochs_val_period
+            Number of epochs to wait before evaluating the validation loss (if a validation
+            `TimeSeries` is passed to the `fit()` method).
+        torch_device_str
+            Optionally, a string indicating the torch device to use. (default: "cuda:0" if a GPU
+            is available, otherwise "cpu")
+        force_reset
+            If set to `True`, any previously-existing model with the same name will be reset (all checkpoints will
+            be discarded).
         """
 
         kwargs['input_chunk_length'] = input_chunk_length
@@ -439,7 +481,10 @@ class NBEATSModel(TorchForecastingModel):
         if isinstance(layer_widths, int):
             self.layer_widths = [layer_widths] * num_stacks
 
-    def _create_model(self, input_dim: int, output_dim: int) -> torch.nn.Module:
+    def _create_model(self, train_sample: Tuple[torch.Tensor]) -> torch.nn.Module:
+        # samples are made of (past_target, past_covariates, future_target)
+        input_dim = train_sample[0].shape[1] + (train_sample[1].shape[1] if train_sample[1] is not None else 0)
+        output_dim = train_sample[-1].shape[1]
 
         return _NBEATSModule(
             input_dim=input_dim,
