@@ -8,14 +8,14 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 from numpy.random import RandomState
-from typing import Optional, Union, Sequence
+from typing import Optional, Union, Sequence, Tuple
 from ..timeseries import TimeSeries
 from ..utils.torch import random_method
-from ..utils.data import ShiftedDataset
+from ..utils.data import PastCovariatesShiftedDataset
 from ..utils.likelihood_models import LikelihoodModel
 
 from ..logging import raise_if_not, get_logger
-from .torch_forecasting_model import TorchParametricProbabilisticForecastingModel
+from .torch_forecasting_model import TorchParametricProbabilisticForecastingModel, PastCovariatesTorchModel
 
 logger = get_logger(__name__)
 
@@ -206,7 +206,7 @@ class _TCNModule(nn.Module):
         return x
 
 
-class TCNModel(TorchParametricProbabilisticForecastingModel):
+class TCNModel(TorchParametricProbabilisticForecastingModel, PastCovariatesTorchModel):
     @random_method
     def __init__(self,
                  input_chunk_length: int,
@@ -226,6 +226,8 @@ class TCNModel(TorchParametricProbabilisticForecastingModel):
         This is an implementation of a dilated TCN used for forecasting.
         Inspiration: https://arxiv.org/abs/1803.01271
 
+        This model supports past covariates (known for `input_chunk_length` points before prediction time).
+
         Parameters
         ----------
         input_chunk_length
@@ -250,6 +252,47 @@ class TCNModel(TorchParametricProbabilisticForecastingModel):
         random_state
             Control the randomness of the weights initialization. Check this
             `link <https://scikit-learn.org/stable/glossary.html#term-random-state>`_ for more details.
+
+        batch_size
+            Number of time series (input and output sequences) used in each training pass.
+        n_epochs
+            Number of epochs over which to train the model.
+        optimizer_cls
+            The PyTorch optimizer class to be used (default: `torch.optim.Adam`).
+        optimizer_kwargs
+            Optionally, some keyword arguments for the PyTorch optimizer (e.g., `{'lr': 1e-3}`
+            for specifying a learning rate). Otherwise the default values of the selected `optimizer_cls`
+            will be used.
+        lr_scheduler_cls
+            Optionally, the PyTorch learning rate scheduler class to be used. Specifying `None` corresponds
+            to using a constant learning rate.
+        lr_scheduler_kwargs
+            Optionally, some keyword arguments for the PyTorch optimizer.
+        loss_fn
+            PyTorch loss function used for training.
+            This parameter will be ignored for probabilistic models if the `likelihood` parameter is specified.
+            Default: `torch.nn.MSELoss()`.
+        model_name
+            Name of the model. Used for creating checkpoints and saving tensorboard data. If not specified,
+            defaults to the following string "YYYY-mm-dd_HH:MM:SS_torch_model_run_PID", where the initial part of the
+            name is formatted with the local date and time, while PID is the processed ID (preventing models spawned at
+            the same time by different processes to share the same model_name). E.g.,
+            2021-06-14_09:53:32_torch_model_run_44607.
+        work_dir
+            Path of the working directory, where to save checkpoints and Tensorboard summaries.
+            (default: current working directory).
+        log_tensorboard
+            If set, use Tensorboard to log the different parameters. The logs will be located in:
+            `[work_dir]/.darts/runs/`.
+        nr_epochs_val_period
+            Number of epochs to wait before evaluating the validation loss (if a validation
+            `TimeSeries` is passed to the `fit()` method).
+        torch_device_str
+            Optionally, a string indicating the torch device to use. (default: "cuda:0" if a GPU
+            is available, otherwise "cpu")
+        force_reset
+            If set to `True`, any previously-existing model with the same name will be reset (all checkpoints will
+            be discarded).
         """
 
         raise_if_not(kernel_size < input_chunk_length,
@@ -271,7 +314,11 @@ class TCNModel(TorchParametricProbabilisticForecastingModel):
         self.dropout = dropout
         self.weight_norm = weight_norm
 
-    def _create_model(self, input_dim: int, output_dim: int) -> torch.nn.Module:
+    def _create_model(self, train_sample: Tuple[torch.Tensor]) -> torch.nn.Module:
+        # samples are made of (past_target, past_covariates, future_target)
+        input_dim = train_sample[0].shape[1] + (train_sample[1].shape[1] if train_sample[1] is not None else 0)
+        output_dim = train_sample[-1].shape[1]
+
         target_size = (
             self.likelihood._num_parameters * output_dim if self.likelihood is not None else output_dim
         )
@@ -288,11 +335,13 @@ class TCNModel(TorchParametricProbabilisticForecastingModel):
 
     def _build_train_dataset(self,
                              target: Sequence[TimeSeries],
-                             covariates: Optional[Sequence[TimeSeries]]) -> ShiftedDataset:
-        return ShiftedDataset(target_series=target,
-                              covariates=covariates,
-                              length=self.input_chunk_length,
-                              shift=self.output_chunk_length)
+                             past_covariates: Optional[Sequence[TimeSeries]],
+                             future_covariates: Optional[Sequence[TimeSeries]]) -> PastCovariatesShiftedDataset:
+
+        return PastCovariatesShiftedDataset(target_series=target,
+                                            covariates=past_covariates,
+                                            length=self.input_chunk_length,
+                                            shift=self.output_chunk_length)
     
     @random_method
     def _produce_predict_output(self, input):
