@@ -65,6 +65,10 @@ class MappersTestCase(unittest.TestCase):
 
         for to_transform, expected_output in test_cases:
             transformed = self.subtract_month.transform(to_transform)
+            if isinstance(to_transform, list):
+                expected_output = [o.with_columns_renamed(o.components[0], t.components[0]) for t, o in zip(transformed, expected_output)]
+            else:
+                expected_output = expected_output.with_columns_renamed(expected_output.components[0], transformed.components[0])
             self.assertEqual(transformed, expected_output)
 
     def test_invertible_mapper_with_timestamp(self):
