@@ -8,8 +8,15 @@ from ..timeseries import TimeSeries
 from ..utils import timeseries_generation as tg
 from ..metrics import mape
 from ..models import (
-    NaiveSeasonal, ExponentialSmoothing, ARIMA, Theta, FourTheta, FFT, VARIMA,
-    RandomForest, LinearRegressionModel
+    NaiveSeasonal,
+    ExponentialSmoothing,
+    ARIMA,
+    Theta,
+    FourTheta,
+    FFT,
+    VARIMA,
+    RandomForest,
+    LinearRegressionModel,
 )
 from ..utils.utils import SeasonalityMode, TrendMode, ModelMode
 from ..logging import get_logger
@@ -30,7 +37,7 @@ models = [
     (FourTheta(trend_mode=TrendMode.EXPONENTIAL), 5.5),
     (FourTheta(model_mode=ModelMode.MULTIPLICATIVE), 11.4),
     (FourTheta(season_mode=SeasonalityMode.ADDITIVE), 14.2),
-    (FFT(trend='poly'), 11.4),
+    (FFT(trend="poly"), 11.4),
     (NaiveSeasonal(), 32.4),
     (LinearRegressionModel(lags=12), 11.0),
     (RandomForest(lags=12, n_estimators=200, max_depth=3), 15.5),
@@ -41,29 +48,31 @@ multivariate_models = [
     (VARIMA(1, 1, 1), 57.0),
 ]
 
-extended_models = [ARIMA(), LinearRegressionModel(lags=4, lags_exog=2), RandomForest(lags=4, lags_exog=2)]
+dual_models = [ARIMA()]
 
 
 try:
     from ..models import Prophet
     models.append((Prophet(), 13.5))
 except ImportError:
-    logger.warning('Prophet not installed - will be skipping Prophet tests')
+    logger.warning("Prophet not installed - will be skipping Prophet tests")
 
 try:
     from ..models import AutoARIMA
+
     models.append((AutoARIMA(), 12.2))
-    extended_models.append(AutoARIMA())
+    dual_models.append(AutoARIMA())
     PMDARIMA_AVAILABLE = True
 except ImportError:
-    logger.warning('pmdarima not installed - will be skipping AutoARIMA tests')
+    logger.warning("pmdarima not installed - will be skipping AutoARIMA tests")
     PMDARIMA_AVAILABLE = False
 
 try:
     from ..models import TCNModel
+
     TORCH_AVAILABLE = True
 except ImportError:
-    logger.warning('Torch not installed - will be skipping Torch models tests')
+    logger.warning("Torch not installed - will be skipping Torch models tests")
     TORCH_AVAILABLE = False
 
 
@@ -78,7 +87,7 @@ class LocalForecastingModelsTestCase(DartsBaseTestClass):
 
     # real timeseries for functionality tests
     ts_passengers = AirPassengersDataset().load()
-    ts_pass_train, ts_pass_val = ts_passengers.split_after(pd.Timestamp('19570101'))
+    ts_pass_train, ts_pass_val = ts_passengers.split_after(pd.Timestamp("19570101"))
 
     # real multivariate timeseries for functionality tests
     ts_ice_heater = IceCreamHeaterDataset().load()
@@ -112,7 +121,7 @@ class LocalForecastingModelsTestCase(DartsBaseTestClass):
 
     def test_multivariate_input(self):
         es_model = ExponentialSmoothing()
-        ts_passengers_enhanced = self.ts_passengers.add_datetime_attribute('month')
+        ts_passengers_enhanced = self.ts_passengers.add_datetime_attribute("month")
         with self.assertRaises(AssertionError):
             es_model.fit(ts_passengers_enhanced)
         es_model.fit(ts_passengers_enhanced["#Passengers"])
@@ -120,31 +129,30 @@ class LocalForecastingModelsTestCase(DartsBaseTestClass):
             es_model.fit(ts_passengers_enhanced["2"])
 
     def test_exogenous_variables_support(self):
-        for model in extended_models:
+        for model in dual_models:
 
             # Test models runnability
-            model.fit(self.ts_gaussian, exog=self.ts_gaussian)
+            model.fit(self.ts_gaussian, future_covariates=self.ts_gaussian)
 
             prediction = model.predict(
                 self.forecasting_horizon,
-                exog=tg.gaussian_timeseries(
+                future_covariates=tg.gaussian_timeseries(
                     length=self.forecasting_horizon,
-                    start_ts=self.ts_gaussian.end_time()+self.ts_gaussian.freq
-                    )
-                )
+                    start_ts=self.ts_gaussian.end_time() + self.ts_gaussian.freq))
+
             self.assertTrue(len(prediction) == self.forecasting_horizon)
 
             # Test mismatch in length between exogenous variables and forecasting horizon
             with self.assertRaises(ValueError):
                 model.predict(
                     self.forecasting_horizon,
-                    exog=tg.gaussian_timeseries(length=self.forecasting_horizon - 1))
+                    future_covariates=tg.gaussian_timeseries(length=self.forecasting_horizon - 1))
 
             # Test mismatch in time-index/length between series and exogenous variables
             with self.assertRaises(ValueError):
-                model.fit(self.ts_gaussian, exog=self.ts_gaussian[:-1])
+                model.fit(self.ts_gaussian, future_covariates=self.ts_gaussian[:-1])
             with self.assertRaises(ValueError):
-                model.fit(self.ts_gaussian[1:], exog=self.ts_gaussian[:-1])
+                model.fit(self.ts_gaussian[1:], future_covariates=self.ts_gaussian[:-1])
 
     def test_dummy_series(self):
         values = np.random.uniform(low=-10, high=10, size=100)
