@@ -4,14 +4,15 @@ Utils for time series generation
 """
 
 import math
-from typing import Union
+
+from typing import Union, Optional
 
 import numpy as np
 import pandas as pd
 import holidays
 
 from ..timeseries import TimeSeries
-from ..logging import raise_if_not, get_logger
+from ..logging import raise_if_not, get_logger, raise_log, raise_if
 
 logger = get_logger(__name__)
 
@@ -19,7 +20,8 @@ logger = get_logger(__name__)
 def constant_timeseries(value: float = 1,
                         length: int = 10,
                         freq: str = 'D',
-                        start_ts: pd.Timestamp = pd.Timestamp('2000-01-01')) -> TimeSeries:
+                        start_ts: pd.Timestamp = pd.Timestamp('2000-01-01'),
+                        column_name: Optional[str] = 'constant') -> TimeSeries:
     """
     Creates a constant univariate TimeSeries with the given value, length, start date and frequency.
 
@@ -44,14 +46,15 @@ def constant_timeseries(value: float = 1,
     times = pd.date_range(periods=length, freq=freq, start=start_ts)
     values = np.full(length, value)
 
-    return TimeSeries.from_times_and_values(times, values, freq=freq)
+    return TimeSeries.from_times_and_values(times, values, freq=freq, columns=pd.Index([column_name]))
 
 
 def linear_timeseries(start_value: float = 0,
                       end_value: float = 1,
                       length: int = 10,
                       freq: str = 'D',
-                      start_ts: pd.Timestamp = pd.Timestamp('2000-01-01')) -> TimeSeries:
+                      start_ts: pd.Timestamp = pd.Timestamp('2000-01-01'),
+                      column_name: Optional[str] = 'linear') -> TimeSeries:
     """
     Creates a univariate TimeSeries with a starting value of `start_value` that increases linearly such that
     it takes on the value `end_value` at the last entry of the TimeSeries. This means that
@@ -79,7 +82,7 @@ def linear_timeseries(start_value: float = 0,
 
     times = pd.date_range(periods=length, freq=freq, start=start_ts)
     values = np.linspace(start_value, end_value, length)
-    return TimeSeries.from_times_and_values(times, values, freq=freq)
+    return TimeSeries.from_times_and_values(times, values, freq=freq, columns=pd.Index([column_name]))
 
 
 def sine_timeseries(value_frequency: float = 0.1,
@@ -88,7 +91,8 @@ def sine_timeseries(value_frequency: float = 0.1,
                     value_y_offset: float = 0.,
                     length: int = 10,
                     freq: str = 'D',
-                    start_ts: pd.Timestamp = pd.Timestamp('2000-01-01')) -> TimeSeries:
+                    start_ts: pd.Timestamp = pd.Timestamp('2000-01-01'),
+                    column_name: Optional[str] = 'sine') -> TimeSeries:
     """
     Creates a univariate TimeSeries with a sinusoidal value progression with a given frequency, amplitude,
     phase and y offset.
@@ -123,14 +127,15 @@ def sine_timeseries(value_frequency: float = 0.1,
     )
     values = f(values)
 
-    return TimeSeries.from_times_and_values(times, values, freq=freq)
+    return TimeSeries.from_times_and_values(times, values, freq=freq, columns=pd.Index([column_name]))
 
 
 def gaussian_timeseries(length: int = 10,
                         freq: str = 'D',
                         mean: Union[float, np.ndarray] = 0.,
                         std: Union[float, np.ndarray] = 1.,
-                        start_ts: pd.Timestamp = pd.Timestamp('2000-01-01')) -> TimeSeries:
+                        start_ts: pd.Timestamp = pd.Timestamp('2000-01-01'),
+                        column_name: Optional[str] = 'gaussian') -> TimeSeries:
     """
     Creates a gaussian univariate TimeSeries by sampling all the series values independently,
     from a gaussian distribution with mean `mean` and standard deviation `std`.
@@ -170,14 +175,15 @@ def gaussian_timeseries(length: int = 10,
     times = pd.date_range(periods=length, freq=freq, start=start_ts)
     values = np.random.normal(mean, std, size=length)
 
-    return TimeSeries.from_times_and_values(times, values, freq=freq)
+    return TimeSeries.from_times_and_values(times, values, freq=freq, columns=pd.Index([column_name]))
 
 
 def random_walk_timeseries(length: int = 10,
                            freq: str = 'D',
                            mean: float = 0.,
                            std: float = 1.,
-                           start_ts: pd.Timestamp = pd.Timestamp('2000-01-01')) -> TimeSeries:
+                           start_ts: pd.Timestamp = pd.Timestamp('2000-01-01'),
+                           column_name: Optional[str] = 'random_walk') -> TimeSeries:
     """
     Creates a random walk univariate TimeSeries, where each step is obtained by sampling a gaussian distribution
     with mean `mean` and standard deviation `std`.
@@ -203,13 +209,63 @@ def random_walk_timeseries(length: int = 10,
     times = pd.date_range(periods=length, freq=freq, start=start_ts)
     values = np.cumsum(np.random.normal(mean, std, size=length))
 
-    return TimeSeries.from_times_and_values(times, values, freq=freq)
+    return TimeSeries.from_times_and_values(times, values, freq=freq, columns=pd.Index([column_name]))
+
+
+def _extend_time_index_until(time_index: Union[pd.DatetimeIndex, pd.RangeIndex],
+                             until: Optional[Union[int, str, pd.Timestamp]],
+                             add_length: int,
+                             ) -> pd.DatetimeIndex:
+
+    if not add_length and not until:
+        return time_index
+
+    raise_if(bool(add_length) and bool(until), "set only one of add_length and until")
+
+    end = time_index[-1]
+    freq = time_index.freq
+
+    if add_length:
+        raise_if_not(add_length >= 0, f"Expected add_length, by which to extend the time series by, "
+                                      f"to be positive, got {add_length}")
+
+        try:
+            end += add_length*freq
+        except pd.errors.OutOfBoundsDatetime:
+            raise_log(ValueError(f"the add operation between {end} and {add_length * freq} will overflow"), logger)
+    else:
+        datetime_index = isinstance(time_index, pd.DatetimeIndex)
+
+        if datetime_index:
+            raise_if_not(isinstance(until, (str, pd.Timestamp)), "Expected valid timestamp for TimeSeries, "
+                                                                 "indexed by DatetimeIndex, "
+                                                                 f"for parameter until, got {type(end)}", logger)
+        else:
+            raise_if_not(isinstance(until, int),  "Expected integer for TimeSeries,"
+                                                  "indexed by RangeIndex, ",
+                                                  f"for parameter until, got {type(end)}", logger)
+
+        timestamp = pd.Timestamp(until) if datetime_index else until
+
+        raise_if_not(timestamp > end, f"Expected until, {timestamp} to lie past end of time index {end}")
+
+        ahead = timestamp - end
+        raise_if_not((ahead % freq) == pd.Timedelta(0), f"End date must correspond with frequency {freq} of the time axis", logger)
+
+        end = timestamp
+
+    new_time_index = pd.date_range(start=time_index[0], end=end, freq=freq)
+    return new_time_index
 
 
 def holidays_timeseries(time_index: pd.DatetimeIndex,
                         country_code: str,
                         prov: str = None,
-                        state: str = None) -> TimeSeries:
+                        state: str = None,
+                        column_name: Optional[str] = 'holidays',
+                        until: Optional[Union[int, str, pd.Timestamp]] = None,
+                        add_length: int = 0,
+                        ) -> TimeSeries:
     """
     Creates a binary univariate TimeSeries with index `time_index` that equals 1 at every index that lies within
     (or equals) a selected country's holiday, and 0 otherwise.
@@ -226,25 +282,37 @@ def holidays_timeseries(time_index: pd.DatetimeIndex,
         The province
     state
         The state
+    until
+        Extend the time_index up until timestamp for datetime indexed series
+        and int for range indexed series, should match or exceed forecasting window.
+    add_length
+        Extend the time_index by add_length, should match or exceed forecasting window.
+        Set only one of until and add_length.
 
     Returns
     -------
     TimeSeries
         A new binary holiday TimeSeries instance.
     """
+
+    time_index = _extend_time_index_until(time_index, until, add_length)
     scope = range(time_index[0].year, (time_index[-1] + pd.Timedelta(days=1)).year)
     country_holidays = holidays.CountryHoliday(country_code, prov=prov, state=state, years=scope)
     index_series = pd.Series(time_index, index=time_index)
     values = index_series.apply(lambda x: x in country_holidays).astype(int)
-    return TimeSeries.from_times_and_values(time_index, values)
+    return TimeSeries.from_times_and_values(time_index, values, columns=pd.Index([column_name]))
 
 
 def datetime_attribute_timeseries(time_index: Union[pd.DatetimeIndex, TimeSeries],
                                   attribute: str,
-                                  one_hot: bool = False) -> TimeSeries:
+                                  one_hot: bool = False,
+                                  cyclic: bool = False,
+                                  until: Optional[Union[int, str, pd.Timestamp]] = None,
+                                  add_length: int = 0) -> TimeSeries:
     """
     Returns a new TimeSeries with index `time_index` and one or more dimensions containing
-    (optionally one-hot encoded) pd.DatatimeIndex attribute information derived from the index.
+    (optionally one-hot encoded or cyclic encoded) pd.DatatimeIndex attribute information derived from the index.
+
 
     Parameters
     ----------
@@ -256,6 +324,16 @@ def datetime_attribute_timeseries(time_index: Union[pd.DatetimeIndex, TimeSeries
     one_hot
         Boolean value indicating whether to add the specified attribute as a one hot encoding
         (results in more columns).
+    cyclic
+        Boolean value indicating whether to add the specified attribute as a cyclic encoding.
+        Alternative to one_hot encoding, enable only one of the two.
+        (adds 2 columns, corresponding to sin and cos transformation)
+    until
+        Extend the time_index up until timestamp for datetime indexed series
+        and int for range indexed series, should match or exceed forecasting window.
+    add_length
+        Extend the time_index by add_length, should match or exceed forecasting window.
+        Set only one of until and add_length.
 
     Returns
     -------
@@ -266,9 +344,13 @@ def datetime_attribute_timeseries(time_index: Union[pd.DatetimeIndex, TimeSeries
     if isinstance(time_index, TimeSeries):
         time_index = time_index.time_index
 
+    time_index = _extend_time_index_until(time_index, until, add_length)
+
     raise_if_not(hasattr(pd.DatetimeIndex, attribute), '"attribute" needs to be an attribute '
                  'of pd.DatetimeIndex', logger)
 
+    raise_if(one_hot and cyclic, "set only one of one_hot or cyclic to true", logger)
+
     num_values_dict = {
         'month': 12,
         'day': 31,
@@ -278,15 +360,30 @@ def datetime_attribute_timeseries(time_index: Union[pd.DatetimeIndex, TimeSeries
     }
 
     values = getattr(time_index, attribute)
-    if one_hot:
+
+    if one_hot or cyclic:
         raise_if_not(attribute in num_values_dict, "Given datetime attribute not supported"
-                                                   " with one-hot encoding.", logger)
+                                                   " with one-hot or cyclical encoding.", logger)
+
+    if one_hot:
         values_df = pd.get_dummies(values)
         # fill missing columns (in case not all values appear in time_index)
         for i in range(1, num_values_dict[attribute] + 1):
             if not (i in values_df.columns):
                 values_df[i] = 0
         values_df = values_df[range(1, num_values_dict[attribute] + 1)]
+    elif cyclic:
+        if attribute == "day":
+            periods = [time_index[i].days_in_month for i in time_index.month]
+            freq = 2*np.pi * np.reciprocal(periods)
+        else:
+            period = num_values_dict[attribute]
+            freq = 2*np.pi/period
+
+        values_df = pd.DataFrame({
+            attribute+"_sin": np.sin(freq * values),
+            attribute+"_cos": np.cos(freq * values)
+        })
     else:
         values_df = pd.DataFrame(values)
     values_df.index = time_index
@@ -295,3 +392,4 @@ def datetime_attribute_timeseries(time_index: Union[pd.DatetimeIndex, TimeSeries
         values_df.columns = [attribute + '_' + str(column_name) for column_name in values_df.columns]
 
     return TimeSeries.from_dataframe(values_df)
+
